<?php
require CERTUMSSL_ROOT.DS.'helper'.DS.'forms'.DS.'certForm.php';
require CERTUMSSL_ROOT.DS.'helper'.DS.'forms'.DS.'certReissueForm.php';
require CERTUMSSL_ROOT.DS.'helper'.DS.'forms'.DS.'tokenForm.php';
require CERTUMSSL_ROOT.DS.'helper'.DS.'forms'.DS.'statementForm.php';
require CERTUMSSL_ROOT.DS.'helper'.DS.'CertumSSLHelper.php';

class CPanelMainController extends AbstractControler
{
    public $clientUsername;
    public $account;
    public $helper;
    public $errors;

    public function __construct($parent, $options = array())
    {
        parent::__construct($parent, $options);
        $this->account = new CertumSSLAccount($this->clientUsername);
        $this->helper  = new CertumSSLHelper();
    }

    public function MainHTMLAction($input, $vars = array())
    {
        $vars['billing_url'] = CertumSSLDriver::config()->billing_url;

        $vars['certs'] = $this->account->getCerts();
        $form          = new FormCreator('cert', $this,
            array(
            'customParams' => array(
                'billingUrl' => $vars['billing_url'],
                'cancel' => false,
            )
        ));

        $domainList = array();

        foreach ($this->account->getDomains() as $domain) {
            $domainList[$domain->domain] = $domain->domain;
        }

        $formParams = array(
            'domainList' => $domainList,
            'mainURL' => $this->helper->full_url($_SERVER)
        );

        $tokenForm       = new tokenForm($input, $form, $formParams);
        $vars['addForm'] = $tokenForm->getForm();

        if ($this->errors && isset($this->errors['general'])) {
            $vars['error'] = $this->errors['general'];
        }

        return array(
            'template' => 'Main',
            'vars' => $vars
        );
    }

    function statementHTMLAction($input, $vars = array())
    {
        $certificate      = new CertumSSLCertificate();
        $apiAccountParams = $this->account->getAPIAccountParamsByVoucher($input['cert_order_token']);
        if ($apiAccountParams['api_url'] == "" || $apiAccountParams['api_login'] == "" || $apiAccountParams['api_password'] == "") {
            $vars['errors']['general'] = array('You are not allowed to order this product.');
            if (CertumSSLDriver::config()->apilogs_turnon_value == 'on') {
                MGAPILogs::addLog("Client ".$this->clientUsername." are not allowed to order product. Used Token:".$input['cert_order_token'],
                    "Undefined product or API account was not found. Please Check if API Account has correctly setted login,password and api url.");
            }
            return $this->MainHTMLAction($input, $vars);
        }
        $vars['statement'] = $this->account->getStatement($input['cert_order_token']);

        if ($this->account->errors || $vars['statement'] == false) {
            $vars['errors'] = $this->account->errors;
            return $this->MainHTMLAction($input, $vars);
        }

        $form = new FormCreator('cert', $this, array(
            'customParams' => array(
                'cancel' => true,
                'cancelUrl' => $input['cert_mainURL'])
        ));

        $statementForm = new statementForm($input, $form);

        $vars['statementForm'] = $statementForm->getForm();

        return array(
            'template' => 'Statement',
            'vars' => $vars
        );
    }

    function certCancelHTMLAction()
    {
        $certificate          = new CertumSSLCertificate();
        $certificate->orderId = $_POST["orderID"];
        $certificate->userId  = $this->clientUsername;
        $certificate->cancelCertOrder($_POST["relid"]);

        if (!empty($certificate->errors)) {
            $errorsArray = $certificate->errors;
            echo "<<JSON RESPONSE START>>";
            echo json_encode(array('status' => 'error', 'errors' => json_encode($errorsArray['cancelOrder'])));
            echo "<<JSON RESPONSE END>>";
            die();
        }

        echo "<<JSON RESPONSE START>>";
        echo json_encode(array('status' => 'success', 'action' => 'CANCEL', 'msg' => 'Certyficate has been cancelled'));
        echo "<<JSON RESPONSE END>>";
        die();
    }

    function certRevokeHTMLAction()
    {
        if ($_POST["serialNumber"]) {
            $certificate         = new CertumSSLCertificate();
            $certificate->userId = $this->clientUsername;
            $certificate->revokeCertOrder($_POST["serialNumber"], $_POST["relid"]);


            if (!empty($certificate->errors)) {
                $errorsArray = $certificate->errors;
                echo "<<JSON RESPONSE START>>";
                echo json_encode(array('status' => 'error', 'errors' => json_encode($errorsArray['revokeCertificate'])));
                echo "<<JSON RESPONSE END>>";
                die();
            }

            echo "<<JSON RESPONSE START>>";
            echo json_encode(array('status' => 'success', 'action' => 'REVOKING', 'msg' => 'Certificate has been revoked'));
            echo "<<JSON RESPONSE END>>";
            die();
        }

        echo "<<JSON RESPONSE START>>";
        echo json_encode(array('status' => 'error', 'errors' => json_encode(array('Missing Certyficate Serial Number'))));
        echo "<<JSON RESPONSE END>>";
        die();
    }

    function certReissueHTMLAction()
    {
        if ($_POST["serialNumber"]) {
            $certificate          = new CertumSSLCertificate();
            $certificate->userId  = $this->clientUsername;
            $certificate->orderId = $_POST["orderid"];
            $certificate->generateCSR(true);
            $certificate->reissueCertyficate($_POST["serialNumber"], $_POST["relid"]);

            if (!empty($certificate->errors)) {
                $errorsArray = $certificate->errors;
                foreach ($errorsArray['reissueCertificate'] as $errorArray) {
                    foreach ($errorArray as $errorParam) {
                        $vars['error'] = $errorParam['text'];
                    }
                }
                return $this->MainHTMLAction($input, $vars);
            }

            $vars['success'] = MGLang::T('orderReissued');
            return $this->MainHTMLAction($input, $vars);
        }

        $vars['error'] = 'Missing Certyficate Serial Number';
        return $this->MainHTMLAction($input, $vars);
    }

    function addCertificateHTMLAction($input, $vars = array())
    {
        $fieldErorrs = isset($vars['fieldErrors']) ? $vars['fieldErrors'] : array();

        $certificate  = new CertumSSLCertificate();
        $productCode  = $this->account->getProductCodeFromToken($input['cert_order_token']);
        $SSLProducts  = new SSLProducts();
        $countDomains = $SSLProducts->getDomainsByProductCode($productCode);
        $domainList   = array();

        foreach ($this->account->getDomains() as $domain) {
            $domainList[$domain->domain] = $domain->domain;
        }

        $sslProductType = $SSLProducts->getProductTypeByCode($productCode);

        $formParams = array('domainList' => $domainList, "maxDomains" => $countDomains, 'productType' => $sslProductType);

        $form     = new FormCreator('cert', $this, array('cancel' => true));
        $certForm = new certForm($input, $form, $fieldErorrs, $formParams);

        $vars['addForm'] = $certForm->getForm();

        if (isset($vars['error']['quickOrder'])) {
            $vars['error'] = $vars['error']['quickOrder'];
        }

        return array(
            'template' => 'AddCert',
            'vars' => $vars
        );
    }

    function submitCertificateHTMLAction($input, $vars = array())
    {
        if (!$input['cert_domain']) {
            $vars['error'] = "You must selecet at last one domain";
            return $this->addCertificateHTMLAction($input, $vars);
        }

        $order = new MGCertOrders(array('_tableName' => 'certumssl_orders'));

        if ($order->getOrderByVoucher($input['cert_order_token'])) {
            return $this->MainHTMLAction($input, $vars);
        }

        $productCode    = $this->account->getProductCodeFromToken($input['cert_order_token']);
        $SSLProducts    = new SSLProducts();
        $sslProductType = $SSLProducts->getProductTypeByCode($productCode);


        $certificate         = new CertumSSLCertificate();
        $certificate->userId = $this->clientUsername;

        if (!is_array($input['cert_domain']) && strpos($sslProductType, "wildcard") !== false) {
            $certificate->domainId = '*.'.$input['cert_domain'];
        } else {
            $certificate->domainId = $input['cert_domain'];
        }

        $certificateInputConditions = $this->helper->certificateInputConditions($input);
        $certificate->orderVoucher  = $input['cert_order_token'];
        $OrganizationName           = null;
        if (isset($input['cert_certdetails_organisation']) && $input['cert_certdetails_organisation'] != '') {
            $OrganizationName = $input['cert_certdetails_organisation'];
        } else {
            $OrganizationName = $input['cert_contact_firstname'].' '.$input['cert_contact_lastname'];
        }


        $certificate->certOrgContacts = $this->helper->certificateInputConditions($input);

        $certificate->certContacts = array(
            'firstName' => $input['cert_contact_firstname'],
            'lastName' => $input['cert_contact_lastname'],
            'address' => $input['cert_contact_address'],
            'city' => $input['cert_contact_city'],
            'country' => $input['cert_contact_country'],
            'email' => $input['cert_contact_email'],
            'phone' => $input['cert_contact_phone'],
            'postalCode' => $input['cert_contact_postalcode'],
            'organisationName' => $input['cert_contactorg_name'],
            'cert_contactorg_name' => $input['cert_contactorg_name'],
            'taxNumber' => $input['cert_contact_taxIdentificationNumber'],
            'verificationPhoneNumber' => $input['cert_org_verifiphonenumber']
        );

        $certificate->certDetails = array(
            'organisation' => $input['cert_certdetails_organisation'],
            'organisation_unit' => $input['cert_certdetails_organisation_unit'],
            'locality' => $input['cert_certdetails_locality'],
            'state_province' => $input['cert_certdetails_state_province'],
            'country' => $input['cert_certdetails_country'],
            'email' => $input['cert_certdetails_email'],
            'serial_number' => $input['cert_certdetails_serial_number'],
            'business_category' => $input['cert_certdetails_business_category'],
            'street_address' => $input['cert_certdetails_street_address'],
            'postal_code' => $input['cert_certdetails_postal_code'],
            'JoILN' => $input['cert_certdetails_JoILN'],
            'JoISoPN' => $input['cert_certdetails_JoISoPN'],
            'JoISoCN' => $input['cert_certdetails_JoISoCN']
        );

        $certificate->taxIdentificationNumber = isset($input['cert_org_taxIdentificationNumber']) ? $input['cert_org_taxIdentificationNumber'] : $input['cert_contact_taxIdentificationNumber'];
        $certificate->verifiPhoneNumber       = isset($input['cert_org_verifiphonenumber']) ? $input['cert_org_verifiphonenumber'] : null;

        if ($errors = $certificate->parseCertyficateData($sslProductType)) {
            $vars['fieldErrors'] = $errors;
            return $this->addCertificateHTMLAction($input, $vars);
        }

        $certificate->generateCSR();
        $certificate->placeCertRequest();

        if (empty($certificate->errors)) {
            $ordeerDetails = $certificate->getOrderDetails();
            $certificate->setOrderStatus($ordeerDetails[0]['orderdefault']['status']);

            $vars['success'] = MGLang::T('orderAddedSuccessfull');
            return $this->MainHTMLAction($input, $vars);
        }

        $vars['error'] = $certificate->errors;
        return $this->addCertificateHTMLAction($input, $vars);
    }

    function getInfoJSONAction($input, $vars = array())
    {
        if (isset($_POST['data']['orderID'])) {
            $certificate          = new CertumSSLCertificate();
            $certificate->orderId = $_POST['data']['orderID'];
            $certificate->userId  = $this->clientUsername;
            $ordersParams         = $certificate->getOrderDetails($input['data']['relid']);

            foreach ($ordersParams as $orderParam) {
                if ($orderParam['orderdefault']['orderID'] == $_POST['data']['orderID']) {
                    $vars = $orderParam;
                }
            }

            $vars['orderdefault']['voucherCode'] = $_POST['data']['voucherCode'];
            $vars['orderdefault']['domainName']  = $_POST['data']['domainName'];
            $vars['orderdefault']['orderID']     = $_POST['data']['orderID'];
            //echo '<pre>';
            //var_dump($vars);die;
            return json_encode($vars);
        }
    }
}