<?php
/* * ********************************************************************
 * Customization Services by ModulesGarden.com
 * Copyright (c) ModulesGarden, INBS Group Brand, All Rights Reserved
 * (2016-01)
 *
 *
 *  CREATED BY MODULESGARDEN       ->        http://modulesgarden.com
 *  CONTACT                        ->       contact@modulesgarden.com
 *
 *
 *
 *
 * This software is furnished under a license and may be used and copied
 * only  in  accordance  with  the  terms  of such  license and with the
 * inclusion of the above copyright notice.  This software  or any other
 * copies thereof may not be provided or otherwise made available to any
 * other person.  No title to and  ownership of the  software is  hereby
 * transferred.
 *
 *
 * ******************************************************************** */
/**
 * @author Grzegorz Draganik <grzegorz@modulesgarden.com>
 * @author Piotr Kozdęba <piotr.ko@modulesgarden.com>
 */
set_time_limit (30);
error_reporting(E_ALL ^ E_NOTICE);
/**
 * DirectAdmin API lib entry point
 * Other classes that make requests should extend it
 */
class DirectAdminApi
{
    public static $total = 0;
    protected $host;
    protected $login;
    protected $pass;
    protected $secure;
    protected $port;
    protected $configurationFilename = 'da_api.conf';

    /**
     * Setups credentials for connection. If host is not supported then it will automatically read credentials from $configurationFilename
     * @param string $host
     * @param string $login
     * @param string $pass
     * @param string $secure - ssl connection [T/F]
     * @param string $port
     */
    public function __construct($host = null, $login = null, $pass = null, $secure = true, $port = 2222)
    {
        $this->host	= $host;
        $this->login	= $login;
        $this->pass	= $pass;
        $this->secure	= $secure;
        $this->port	= $port;
        if (empty($host))
        {
            $this->setupLocalConnection();
        }
    }

    /**
     * Gets system information
     * Values:
     *  numcpus= int, number of cpus on the box
     *  0=mhz=1662.519&model_name=AMD Sempron(tm) 2400 &vendor_id=AuthenticAMD
     *  (1=mhz..etc.. for multiple CPUs)
     *
     *  MemTotal= number in human readible format with Kb/Mb, etc.
     *  MemFree=
     8  SwapTotal=
     *  SwapFree=
     *  uptime=42 Days, 23 Hours and 43 Minutes
     *
     *  directadmin=1.33.7|Running
     *  proftpd=1.3.2|*** Stopped ***
     *  etc.. for each service
     * @return array
     * @throws Exception
     */
    public function getSystemInfo()
    {
        $sock = $this->getSocket();
        $sock->query('/CMD_API_SYSTEM_INFO');
        $result = $sock->fetch_body();
        if (!$result || strpos($result, '<form'))
        {
            throw new Exception('Unable to get system info');
        }
        $details = array();
        parse_str(substr($result, 2), $details);
        return $details;
    }

    /**
     * Setups credentials for local connection. It will automatically read credentials from $configurationFilename
     */
    protected function setupLocalConnection()
    {
        // THAT COMMENTED OUT WILL NOT WORK IN HOOKS -> all connection details are in directadminapi.conf
        //		if (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443)
        //			$this->_secure = true;
        //		else
        //			$this->_secure = false;
        //
        //		$this->_host	= $_SERVER['SERVER_NAME'];
        //		$this->_port	= $_SERVER['SERVER_PORT'];
        $conf = new DirectAdminConfiguration('da_api.conf');
        $this->login	= $conf->get('username');
        $this->pass	= $conf->get('password');
        $this->secure	= (bool)$conf->get('secure');
        $this->host	= $conf->get('host');
        $this->port	= $conf->get('port');
    }

    /**
     * Tests api connection (non-socket)
     * @return array
     */
    public function testConnection()
    {
        $out = $this->getRequest('CMD_API_SHOW_USER_USAGE', array('user' => $this->oconfig['username']));
        if($out === false)
        {
            return array(
                'status' => false,
                'error' => $this->error
            );
        }
        elseif(strpos($out, '<html>') !== false)
        {
           return array(
                'status' => false,
                'error' => 'Incorrect username or password'
            );
        }
        else
        {
            return array(
                'status' => true,
                'error' => ''
            );
        }
    }

    /**
     * Calls HTTP api request (not-socket)
     * @param string $function - api request to call
     * @param array $params - api parameters
     * @param string $type - request type (post/get/delete/...)
     * @return boolean
     */
    private function request($function, $params=array(), $type='GET')
    {
        $username = urlencode($this->login);
        $password = urlencode($this->password);
        $url = ($this->secure?'https://':'http://').$username.':'.$password.'@'.$this->host.':2222'.'/'.$function;
        if(is_array($params))
        {
            $url .= '?';
            foreach($params as $key=>$value)
            {
                $value = urlencode($value);
                $key = urlencode($key);
                $url .= "{$key}={$value}&";
            }
        }
        $ch = curl_init();
        $chOptions = array (
            CURLOPT_URL => trim($url, '&'),
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_SSL_VERIFYHOST => false,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_CUSTOMREQUEST, $type
        );
        curl_setopt_array($ch, $chOptions);
        $out = curl_exec($ch);

        if($out === false)
        {
            $this->error = ucwords(curl_error($ch));
            return false;
        }

        //parsujemy
        $out = urldecode($out);
        parse_str($out, $parsed);

        if(isset($parsed['error']))
        {
            if($parsed['error'] > 0)
            {
                $this->results[] = array('success' => 0, 'info' => 'Error occured:'.$parsed['text'].' '.$parsed['details']);
                return false;
            }
            else
            {
                $this->results[] = array('success' => 1, 'info' => 'Info:'.$parsed['text']);
                return true;
            }
        }
        return $out;
    }

    /**
     * Calls HTTP api request (not-socket; GET)
     * @param string $function - api request to call
     * @param array $params - api parameters
     * @return boolean
     */
    private function getRequest($function,$params)
    {
        return $this->request($function, $params, 'GET');
    }

    /**
     * Calls HTTP api request (not-socket; POST)
     * @param string $function - api request to call
     * @param array $params - api parameters
     * @return boolean
     */
    private function postRequest($function,$params)
    {
        return $this->request($function, $params, 'POST');
    }

    /**
     * Calls HTTP api request (not-socket; DELETE)
     * @param string $function - api request to call
     * @param array $params - api parameters
     * @return boolean
     */
    private function deleteRequest($function,$params)
    {
        return $this->request($function, $params, 'DELETE');
    }

    /**
     * Starts socket connection
     * @return HTTPSocket
     */
    protected function getSocket()
    {
        $sock = new DirectAdminHTTPSocket;
        $sock->connect( ($this->secure ? "ssl://" : '') . $this->host, $this->port);
        $sock->set_login($this->login, $this->pass);
        return $sock;
    }

}