<?php
/* * ********************************************************************
 * Customization Services by ModulesGarden.com
 * Copyright (c) ModulesGarden, INBS Group Brand, All Rights Reserved 
 * (2016-01)
 * 
 *
 *  CREATED BY MODULESGARDEN       ->        http://modulesgarden.com
 *  CONTACT                        ->       contact@modulesgarden.com
 *
 *
 *
 *
 * This software is furnished under a license and may be used and copied
 * only  in  accordance  with  the  terms  of such  license and with the
 * inclusion of the above copyright notice.  This software  or any other
 * copies thereof may not be provided or otherwise made available to any
 * other person.  No title to and  ownership of the  software is  hereby
 * transferred.
 *
 *
 * ******************************************************************** */

/**
 * @author Grzegorz Draganik <grzegorz@modulesgarden.com>
 * @author Piotr Kozdęba <piotr.ko@modulesgarden.com>
 */

/**
 * DirectAdmin API - Domains Management (methods that is not for specific domain - do not nedds passwords/usernames)
 */
class DirectadminApiDomainsDriver extends DirectadminApiDriver
{

    public function getEmail($userName)
    {
        $sock   = $this->getSocket();
        $sock->query('/CMD_API_SHOW_USER_CONFIG?user='.$userName);
        $result = $sock->fetch_body();

        $records  = array();
        $lines    = explode("&", urldecode($result));
        $accounts = array();
        foreach ($lines as $line) {
            if (strtok($line, '=') == 'email') {
                parse_str(substr($line, strlen($recordType) + 1), $details);
                return $details['mail'];
            }
        }
    }

    /**
     * Gets raw zone file for desired domain
     */
    public function getDomainsMxRecords($domain)
    {
        $sock   = $this->getSocket();
        $sock->query('/CMD_API_DNS_ADMIN?domain='.$domain.'&urlencoded=yes');
        $result = $sock->fetch_body();

        if (!$result) {
            throw new Exception('Unable to get DNS zone of domain '.$domain);
        }

        $records = array();
        $lines   = explode("\n", $result);
        foreach ($lines as $line) {
            $recordType = strtok($line, '=');
            if ($recordType != 'MX') {
                continue;
            }

            $details = array();
            parse_str(substr($line, strlen($recordType) + 1), $details);
            foreach ($details as $record => $v) {
                $fixedRecord = str_replace('_', '.', $record);
                $records[]   = array(
                    'original' => $fixedRecord,
                    'full' => substr($fixedRecord, -1) == '.' ? substr($fixedRecord, 0, -1) : $fixedRecord.'.'.$domain,
                    'isRemote' => substr($fixedRecord, -1) == '.' && strpos($fixedRecord, $domain) === false ? true : false
                );
            }
        }
        return $records;
    }

    /**
     * Gets dns object for desired domain
     * @param type $domain
     * @return DNS
     */
    public function getDns($domain)
    {
        $dns = new DirectadminApiDNSDriver(
            $domain, $this->host, $this->login, $this->pass, $this->secure, $this->port
        );
        $dns->setConfig(array(
            'end_dot' => 1,
        ));
        return $dns;
    }

    /**
     * Returns true/false if desired domain is User domain or reseller domain if second parameter is true
     * @param string $domain
     * @param bool $reseller
     * @return boolean
     */
    public function isUsersDomain($domain, $reseller = false)
    {
        $domains = $this->getAllDomains(array(
            'page' => 1,
            'limit' => 999999,
            'searchUsername' => $reseller ? null : getenv('USERNAME'),
            'searchReseller' => $reseller ? getenv('USERNAME') : null,
            'searchDomain' => $domain,
        ));
        foreach ($domains as $d) {
            if ($d->domain == $domain) {
                return true;
            }
        }
        return false;
    }

    /**
     * Gets all domains list, optionally filtered (allowed keys in array (parameter):page/searchUsername/searchReseller/searchDomain/includePointers - values will be searched). Returns Domain ojects 
     * @param array $filters
     * @return Domain
     * @throws Exception
     */
    public function getAllDomains(array $filters = array())
    {
        $sock   = $this->getSocket();
        $sock->query('/CMD_API_ALL_USER_USAGE');
        $result = $sock->fetch_body();

        if (!$result) {
            throw new Exception('Unable to get domains list');
        }

        $startIndex      = 0;
        $endIndex        = 10000;
        $searchReseller  = null;
        $searchUsername  = null;
        $searchDomain    = null;
        $includePointers = false;

        if (isset($filters['page']) && isset($filters['limit']) && $filters['page'] && $filters['limit']) {
            $startIndex = ($filters['page'] - 1) * $filters['limit'];
            $endIndex   = $startIndex + $filters['limit'];
        }
        if (isset($filters['searchUsername']) && $filters['searchUsername']) {
            $searchUsername = $filters['searchUsername'];
        }
        if (isset($filters['searchReseller']) && $filters['searchReseller']) {
            $searchReseller = $filters['searchReseller'];
        }
        if (isset($filters['searchDomain']) && $filters['searchDomain']) {
            $searchDomain = $filters['searchDomain'];
        }
        if (isset($filters['includePointers']) && $filters['includePointers']) {
            $includePointers = true;
        }

        self::$total = 0;
        $domains     = array();
        $lines       = explode("\n", $result);
        foreach ($lines as $line) {
            $username = strtok($line, '=');
            $details  = array();
            parse_str($line, $details);
            if (isset($details['list']) && $details['list']) {
                $ds = explode('<br>', trim($details['list']));
                foreach ($ds as $dom) {
                    if (!$dom) {
                        continue;
                    }
                    if ($searchDomain && strpos($dom, $searchDomain) === false) {
                        continue;
                    }
                    if ($searchUsername && strpos($username, $searchUsername) === false) {
                        continue;
                    }
                    if ($searchReseller && strpos($details['creator'], $searchReseller) === false) {
                        continue;
                    }

                    self::$total++;
                    $domains[$dom] = array(
                        'username' => $username,
                    );
                }
            }
            foreach ($details as $detailkey => $detail) {
                if (substr($detailkey, 0, 8) == 'pointer_') {
                    $pointers = explode('|', $detail);
                    foreach ($pointers as $pointer) {
                        if ($pointer) {
                            if ($searchDomain && strpos($pointer, $searchDomain) === false) {
                                continue;
                            }
                            if ($searchUsername && strpos($username, $searchUsername) === false) {
                                continue;
                            }
                            if ($searchReseller && strpos($details['creator'], $searchReseller) === false) {
                                continue;
                            }

                            self::$total++;
                            $domains[$pointer] = array(
                                'username' => $username,
                                'pointsto' => str_replace('_', '.', substr($detailkey, 8))
                            );
                        }
                    }
                }
            }
        }

        ksort($domains);

        $di     = 0;
        $return = array();
        foreach ($domains as $domain => $details) {
            if ($di >= $startIndex && $di <= $endIndex) {
                $return[$domain] = new DirectadminApiDomainDriver(
                    $domain, $details['username'], isset($details['pointsto']) ? $details['pointsto'] : null, $this->host, $this->login, $this->pass, $this->secure, $this->port
                );
            }
            $di++;
        }

        return $return;
    }

    /**
     * Saves SSL for desired domains. Returns array with 2 values: text and details
     * @param type $domain
     * @param type $private_key
     * @param type $certificate
     */
    public function saveSSL($user, $domain, $private_key, $certificate)
    {
        $sock = $this->getSocket($user);
        $sock->set_method('POST');

        $certificate_prepared = '
'.$private_key.'
'.$certificate.'
';
        $sock->query('/CMD_API_SSL',
            array(
            'domain' => $domain,
            'action' => 'save',
            'type' => 'paste',
            'certificate' => $certificate_prepared
            )
        );

        $result = $sock->fetch_parsed_body();

        if ($result["key"] && $private_key == '') {
            $certificate_prepared = '
'.trim($result["key"]).'
'.$certificate.'
';
            $sock->query('/CMD_API_SSL',
                array(
                'domain' => $domain,
                'action' => 'save',
                'type' => 'paste',
                'certificate' => $certificate_prepared
                )
            );
            $result               = $sock->fetch_parsed_body();
        }


        return array(
            'text' => isset($result['text']) ? $result['text'] : '',
            'details' => isset($result['details']) ? $result['details'] : '',
            'error' => isset($result['error']) ? $result['error'] : '',
        );
    }

    public function getCertificate($domain)
    {
        $sock = $this->getSocket($_SERVER['USERNAME']);
        $sock->set_method('GET');

        $sock->query('/CMD_API_SSL?domain='.$domain, array());
        $result = $sock->fetch_parsed_body();
        return $result['certificate'];
    }

    public function getDomainCertyficateKey($domain)
    {
        $sock = $this->getSocket($_SERVER['USERNAME']);
        $sock->set_method('GET');

        $sock->query('/CMD_API_SSL?domain='.$domain, array());
        $result = $sock->fetch_parsed_body();
        if (isset($result['key'])) {
            return $result['key'];
        }
        return "";
    }

    public function generateSSL($country, $province, $city, $company, $division = '', $domain, $email, $forReissue = false)
    {
        //Czit do generowania nowego private keya
        if ($forReissue) {
            $sock = $this->getSocket($_SERVER['USERNAME']);
            $sock->set_method('POST');
            $sock->query('/CMD_API_SSL?domain='.str_replace("*.", "", $domain),
                array(
                'action' => 'save',
                'type' => 'create',
                'request' => 'no',
                'country' => $country,
                'province' => $province,
                'city' => $city,
                'company' => preg_replace('/[^a-zA-Z ]/', "", $company),
                'division' => $division,
                'name' => $domain,
                'email' => $email,
                'keysize' => 2048,
                'encryption' => 'sha256'
                )
            );
        }


        $sock = $this->getSocket($_SERVER['USERNAME']);
        $sock->set_method('POST');
        $sock->query('/CMD_API_SSL?domain='.str_replace("*.", "", $domain),
            array(
            'action' => 'save',
            'type' => 'create',
            'request' => 'yes',
            'country' => $country,
            'province' => $province,
            'city' => $city,
            'company' => preg_replace('/[^a-zA-Z ]/', "", $company),
            'division' => $division,
            'name' => $domain,
            'email' => $email,
            'keysize' => 2048,
            'encryption' => 'sha256'
            )
        );

        $result = $sock->fetch_parsed_body();
        $result = preg_split("/&#45\n&#45/", $result['request']); //podzielenie na csr i key

        if (count($result) > 1) {
            $csr = html_entity_decode(preg_replace("/(&#\\d+)/", '$1;', $result[0])).'-';
            $key = "-".html_entity_decode(preg_replace("/(&#\\d+)/", '$1;', $result[1]));
        } else {
            $csr = html_entity_decode(preg_replace("/(&#\\d+)/", '$1;', $result[0]));
            $key = $this->getDomainCertyficateKey(str_replace("*.", "", $domain));
        }


        return array(
            'csr' => $csr,
            'key' => $key
        );
    }
}