<?php
/*
 * * ******************************************************************
 *
 *   CREATED BY MODULESGARDEN       ->        http://modulesgarden.com
 *   AUTHOR                         ->     dariusz.bi@modulesgarden.com
 *   CONTACT                        ->       contact@modulesgarden.com
 *
 *  This software is furnished under a license and may be used and copied
 *  only  in  accordance  with  the  terms  of such  license and with the
 *  inclusion of the above copyright notice.  This software  or any other
 *  copies thereof may not be provided or otherwise made available to any
 *  other person.  No title to and  ownership of the  software is  hereby
 *  transferred.
 *
 * * ******************************************************************
 */

class PleskLocalAPIDriver extends AbstractLocalAPIDriver
{
    protected $apiKey;
    private $sortCol        = 'domain';
    private $sortVect       = 'ASC';
    public $currentUsername = false;
    public $currentDomain   = false;
    public $lastParams      = array();
    public $typeList        = array('base', 'addon', 'parked');

    function __construct($params)
    {
        parent::__construct($params);
    }

    public function _connect($params)
    {

    }

    function enableSSL($domain)
    {
        $request         = '<site>
                        <set>
                            <filter>
                                <name>'.$domain.'</name>
                            </filter>
                            <values>
                                   <hosting>
                                        <vrt_hst>
                                            <property>
                                                <name>ssl</name>
                                                <value>true</value>
                                            </property>
                                        </vrt_hst>
                                    </hosting>
                            </values>
                        </set>
                    </site>';
        $enableSSL       = pm_ApiRpc::getService()->call($request, 'admin')->site->set->result;
        $enableSSLStatus = (array) $enableSSL->status;

        if ($enableSSLStatus[0] == 'ok') {
            return true;
        }
        $error = (array) $enableSSL->errtext;
        return $error[0];
    }

    function installCrt($domain, $csr, $key, $crt, $crt_ca = null, $orderId)
    {
        $request = '<certificate>
                        <install>
                            <name>'.$domain.'-'.$orderId.'</name>
                            <webspace>'.$domain.'</webspace>
                            <content>
                                <csr>'.$csr.'</csr>
                                <pvt>'.$key.'</pvt>
                                <cert>'.$crt.'</cert>';
        if ($crt_ca != null) {
            $request .= '<ca>'.$crt_ca.'</ca>';
        }
        $request .= '</content>
                        </install>
                    </certificate>';

        $installResponse = pm_ApiRpc::getService()->call($request, 'admin')->certificate->install->result;
        $installStatus   = (array) $installResponse->status;
        if ($installStatus[0] == 'ok') {
            return true;
        }
        $error = (array) $installResponse->errtext;
        return $error[0];
    }

    //Generate CSR
    function generateCsr($params = null)
    {
        $request = '<certificate>
            <generate>
               <info>
                  <bits>2048</bits>
                  <country>'.$params['country'].'</country>
                  <state>'.$params['state'].'</state>
                  <location>'.$params['address1'].'</location>
                  <company>'.$params['OrganizationName'].'</company>';

        if (isset($params['organisation_unit']) && $params['organisation_unit'] != "") {
            $request .= '<dept>'.$params['organisation_unit'].'</dept>';
        }

        $request .= '<email>'.$params['email'].'</email>
                  <name>'.$params['domain'].'</name>
               </info>
            </generate>
        </certificate>';

        $csrResponse = pm_ApiRpc::getService()->call($request, 'admin')->certificate->generate->result;

        $csrStatus = (array) $csrResponse->status;

        $details = array(
            'country' => $params['country'],
            'state' => $params['state'],
            'address1' => $params['address1'],
            'OrganizationName' => $params['OrganizationName'],
            'organisation_unit' => $params['organisation_unit'],
            'email' => $params['email'],
            'domain' => $params['domain']
        );

        if ($csrStatus[0] == 'ok') {
            $csr = (array) $csrResponse->csr;
            $pvt = (array) $csrResponse->pvt;

            return array(
                'csr' => $csr[0],
                'key' => $pvt[0],
                'crt' => null,
                'details' => $details
            );
        }

        $response = (array) $csrResponse->result;

        if ($response['status'] == "error") {

            return array('error' => $response['errtext']);
        }
    }

    public function getAccounts()
    {

    }

    function getDomainCertyficate($domainID, $orderid = null)
    {

        $orderid = str_replace("_", "-", $orderid);
        $name    = preg_replace('/[^A-Za-z0-9\-]/', '', $orderid);
        if (!is_numeric($domainID)) {
            $domainName   = $domainID;
            $domainParams = $this->getUserDomainIdByName($domainID);
            if ($domainParams['parentDomainId'] != "" && $domainParams['parentDomainId'] != "0") {
                $domainID = $domainParams['parentDomainId'];
            } else {
                $domainID = $domainParams['id'];
            }
        }

        $request = '<certificate>
                            <get-pool>
                                <filter>
                                    <domain-id>'.$domainID.'</domain-id>
                                </filter>
                            </get-pool>
                    </certificate>';

        $cert = (array) pm_ApiRpc::getService()->call($request, 'admin')->certificate;
        $list = (array) $cert['get-pool']->result->certificates;

        foreach ($list['certificate'] as $certName) {
            $contener = (array) $certName;
            if ($contener['name'] == $domainName.'-'.$orderid) {
                return true;
            }
        }

        return false;
    }

    function getUserDomains($user = null, $main = null, $userid = null)
    {

        $domainsList = array();

        $q = MGMySQL::query("
            SELECT 
                d.id        as domainid,
                d.name      as domain,
                u.id        as userid,
                u.login     as username
            FROM
                domains d
            LEFT JOIN
                clients u 
            ON 
                d.cl_id = u.id
            WHERE 
                u.id = ".(int) $userid."
            ORDER BY
                d.name ASC  
        ")->fetchAll();

        foreach ($q as $r) {
            $domainsList[] = array(
                'domain' => $r['domain']
                , 'domainid' => $r['domainid']
                , 'userid' => $r['userid']
            );
        }

        return $domainsList;
    }

    function getUserDomainIdByName($name)
    {
        $q = MGMySQL::query("
            SELECT
                *
            FROM
                domains
            WHERE 
                name = '".$name."'
            ORDER BY
                name ASC
            LIMIT 1
        ")->fetch();

        return $q;
    }

    function getUserDomainById($domainId)
    {

        $q = MGMySQL::query("
            SELECT 
                *
            FROM
                domains
            WHERE 
                id = '".$domainId."'
            ORDER BY
                name ASC 
            LIMIT 1
        ")->fetch();

        return $q;
    }

    function getDomainRootPath($domainId)
    {
        $q = MGMySQL::query("
            SELECT 
                www_root
            FROM
                hosting
            WHERE 
                dom_id = '".$domainId."'
            ORDER BY
                dom_id ASC 
            LIMIT 1
        ")->fetch();

        return $q['www_root'];
    }

    function clearConfiguration()
    {
        MGMySQL::query("
            DELETE
            FROM 
                certumssl_configuration
            WHERE 
                name <> 'apilogs_turnon_value'
            AND 
               name <>  'email'
            AND 
               name <>  'sender'
            AND 
               name <>  'success_email_subject'
            AND 
               name <>  'success_email_message'
            AND 
               name <>  'failure_email_subject'
            AND 
               name <>  'failure_email_message'
        ");
    }

    function getUserEmailById($userID)
    {

        $q = MGMySQL::query("
            SELECT 
                email
            FROM
                clients
            WHERE 
                id = ".(int) $userID."
            ORDER BY
                email ASC  
            LIMIT 1
        ")->fetch();

        return $q['email'];
    }

    function getUserNameById($userID)
    {

        $q = MGMySQL::query("
            SELECT 
                pname
            FROM
                clients
            WHERE 
                id = ".(int) $userID."
            ORDER BY
                pname ASC  
            LIMIT 1
        ")->fetch();
        return $q['pname'];
    }
}