<?php
/* 
 * * ******************************************************************
 *
 *   CREATED BY MODULESGARDEN       ->        http://modulesgarden.com
 *   AUTHOR                         ->     dariusz.bi@modulesgarden.com
 *   CONTACT                        ->       contact@modulesgarden.com
 *
 *  This software is furnished under a license and may be used and copied
 *  only  in  accordance  with  the  terms  of such  license and with the
 *  inclusion of the above copyright notice.  This software  or any other
 *  copies thereof may not be provided or otherwise made available to any
 *  other person.  No title to and  ownership of the  software is  hereby
 *  transferred.
 *
 * * ******************************************************************
 */

class WHMLocalAPIDriver extends AbstractLocalAPIDriver
{
    protected $apiKey;
    private $sortCol        = 'domain';
    private $sortVect       = 'ASC';
    public $currentUsername = false;
    public $currentDomain   = false;
    public $lastParams      = array();
    public $typeList        = array('base','addon','parked');
        
    function __construct($params) 
    {
        parent::__construct($params);
    }

    public function _connect($params)
    {
        $accesKeyFile = '/root/.accesshash';
        
        if(!file_exists($accesKeyFile)){
            die("Cant find Acces Hash File");
        }
        
        $handle = fopen($accesKeyFile,'r');
        $this->apiKey = preg_replace("'(\r|\n)'","",fread($handle, 1024));
        
        if(empty($this->apiKey)){
            die("Cant find Acces Hash File");
        }
    }

    private function _request($function,array $params = array())
    {
        $ch = curl_init();
        
        $this->lastParams = $params;
       
        curl_setopt($ch, CURLOPT_URL, "http://127.0.0.1:2086/json-api/".$function.'?'.http_build_query($params));
        curl_setopt($ch, CURLOPT_HEADER, 0);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $header[0] = "Authorization: WHM root:" . $this->apiKey;
        # Remove newlines from the hash
        curl_setopt($ch,CURLOPT_HTTPHEADER,$header);

        curl_setopt($ch, CURLOPT_TIMEOUT, 400);
        $data = curl_exec($ch);
        curl_close($ch);
        
        $response = json_decode($data,TRUE);
                        
        return $response;
    }
    
    protected function _userRequest($user,$module,$function,$params = array())
    {
        $params['cpanel_jsonapi_user']     = $user;
        $params['cpanel_jsonapi_module']   = $module;
        $params['cpanel_jsonapi_func']     = $function;
  
        $response = $this->_request('cpanel', $params);

        return $response;
    }
    
    function getMainDomain($account)
    {
        $main = $this->_userRequest($account, 'DomainLookup', 'getmaindomain');
        return $main['cpanelresult']['data'][0]['main_domain'];
    }
    
    function getDomainRoot($account,$params)
    {
        $domainMain = $this->_userRequest($account, 'DomainLookup', 'getdocroot',$params);
        return $domainMain['cpanelresult']['data'][0]['docroot'];
    }
       
    public function installCrt($account,$params)
    { 
        $result = $this->_userRequest($account,'SSL','installssl',$params); 
        return $result;
    }
    
    public function getAccounts()
    {
          $accounts = $this->_request('listaccts');
          $output = array();
          foreach ($accounts['acct'] as $accountId => $account){
              $output[] = array(
                  'name'        => $account['user']
                  ,'email'      => $account['email']
                  ,'locked'     => $account['is_locked']
                  ,'mainDomain' => $account['domain']
              );
          }
          return $output;
    }
    
    function clearConfiguration()
    {
        MGMySQL::query("
            DELETE
            FROM 
                certumssl_configuration
            WHERE 
                name <> 'apilogs_turnon_value'
            AND 
               name <>  'email'
            AND 
               name <>  'sender'
            AND 
               name <>  'success_email_subject'
            AND 
               name <>  'success_email_message'
            AND 
               name <>  'failure_email_subject'
            AND 
               name <>  'failure_email_message'
        ");
    }
    
    function getUserDomains($user = null, $main = null)
    {
        $domainsList = array();

        if($main == null)
        {
            $main = $this->getMainDomain($user);
        }

        $domainsList[] = array(
            'domain'    => $main
            ,'user'     => $user
            ,'type'     => 'base'
            ,'root'     => $main
        );

        $addons = $this->_userRequest($user, 'AddonDomain', 'listaddondomains');
        if(!empty($addons['cpanelresult']['data']) && is_array($addons['cpanelresult']['data']))
        {
            foreach($addons['cpanelresult']['data'] as $addonDomain)
            {
                $domainsList[] = array(
                    'domain'    => $addonDomain['domain']
                    ,'user'     => $user
                    ,'type'     => 'addon'
                    ,'root'     => $addonDomain['rootdomain']
                );
            }
        }
       
        return $domainsList;
    }
}