<?php
/*
 * * ******************************************************************
 *
 *   CREATED BY MODULESGARDEN       ->        http://modulesgarden.com
 *   AUTHOR                         ->     dariusz.bi@modulesgarden.com
 *   CONTACT                        ->       contact@modulesgarden.com
 *
 *  This software is furnished under a license and may be used and copied
 *  only  in  accordance  with  the  terms  of such  license and with the
 *  inclusion of the above copyright notice.  This software  or any other
 *  copies thereof may not be provided or otherwise made available to any
 *  other person.  No title to and  ownership of the  software is  hereby
 *  transferred.
 *
 * * ******************************************************************
 */

class CertumSSLAccount extends abstractModel
{
    public $userId;
    public $nameID;
    public $mainDomain;
    public $relid;
    public $errors   = array();
    private $domains = array();
    private $certs   = array();
    private $client;

    function __construct($userId)
    {
        $this->userId = $userId;
        /*
          $connectionDetails = array(
          "api_login" => CertumSSLDriver::config()->api_login,
          "api_password" => CertumSSLDriver::config()->api_password,
          "api_url" => CertumSSLDriver::config()->api_wsdl_url,
          "api_wsdl_lang" => CertumSSLDriver::config()->api_wsdl_lang,
          "test_mode" => CertumSSLDriver::config()->test_mode
          );
         *
         */
        $this->client = CertumSSLDriver::remoteAPI();
        //$this->client->setConnectionParams($connectionDetails);
    }

    function setConnectionParams($productCode = null, $relid = null)
    {
        $configuration = new CertumSSLConfiguration();

        if ($productCode == null && $relid == null) {
            $connectionDetails = $configuration->getFirstAccountParams();
        } elseif ($relid != null) {
            $connectionDetails = $configuration->getAPIAccountByRelId($relid);
        } else {
            $SSLProducts       = new SSLProducts();
            $productType       = $SSLProducts->getProductTypeByCode($productCode);
            $connectionDetails = $configuration->getAccountByPrductType($productType);
        }

        $this->relid = $connectionDetails['relid'];
        //var_dump($connectionDetails);
        $this->client->setConnectionParams($connectionDetails);
    }

    function getDomains()
    {
        if (empty($this->domains)) {
            foreach (CertumSSLDriver::localAPI()->getUserDomains(null, null, $this->userId) as $domain) {
                $domain = (array) $domain;
                if (isset($domain['domainid'])) {
                    $this->domains[] = new CertumSSLDomain($domain['domain'], $domain['domainid']);
                } else {
                    $this->domains[] = new CertumSSLDomain($domain['domain']);
                }
            }
        }
        return $this->domains;
    }

    function getCerts()
    {
        if (empty($this->certs)) {
            $certumSSLOrders = new MGCertOrders(array('_tableName' => 'certumssl_orders', 'userId' => $this->userId));

            $userOrdersList = $certumSSLOrders->getUserOrders();
            $certs          = array();

            foreach ($userOrdersList as $orderParams) {
                $orderDetails = $this->orderDetails($orderParams);

                if (!$orderDetails && empty($this->errors['orderDetails'][0])) {
                    return false;
                }


                //Dla Rodzicow
                if ($orderParams['finished'] == '1') {
                    $orderParams['status'] = 'Installed';
                }

                //Lecimy po dzieciach
                foreach ($orderDetails as $orderDetail) {
                    if (isset($orderDetail['orderdefault']['orderID'])) {
                        $orderParams['orderid'] = $orderDetail['orderdefault']['orderID'];
                    }
                    $orderParams['status'] = 'Not Installed';

                    $orderParams['reissued'] = $orderDetail['orderdefault']['reissue'];
                    if (isset($orderDetail['cert']['serialNumber'])) {
                        $orderParams['certSerialNumber'] = $orderDetail['cert']['serialNumber'];
                    }

                    if (isset($orderDetail['cert']['certificateStatus']) && $orderDetail['orderdefault']['status'] != 'REJECTED') {
                        $orderParams['cert_status'] = $orderDetail['cert']['certificateStatus'];
                    } else {
                        $orderParams['cert_status'] = $orderDetail['orderdefault']['status'];
                    }
                    if (isset($orderDetail["cert"])) {
                        if (CertumSSLDriver::localAPI()->getDomainCertyficate($orderParams['domainid'], $orderDetail['orderdefault']['orderID'])) {
                            $orderParams['status'] = 'Installed';
                        }
                    }

                    $SSLProducts = new SSLProducts();
                    $productCode = $this->getProductCodeFromToken($orderParams['token']);
                    $certName    = $SSLProducts->getProductTypeByCode($productCode);
                    $certName    = ucwords(str_replace("_", " ", $certName));
                    if (strpos($certName, 'Ssl')) {
                        $certName = str_replace("Ssl", "SSL", $certName);
                    }
                    $orderParams['certName'] = ucwords(str_replace("_", " ", $certName));

                    if (isset($orderDetail['cert']['startDate'])) {
                        $orderParams['startDate'] = date("Y-m-d H:i:s", strtotime($orderDetail['cert']['startDate']));
                    }
                    if (isset($orderDetail['cert']['endDate'])) {
                        $orderParams['endDate'] = date("Y-m-d H:i:s", strtotime($orderDetail['cert']['endDate']));
                    }

                    if (isset($orderDetail['cert']['revokedDate'])) {
                        $orderParams['revokedDate'] = date("Y-m-d H:i:s", strtotime($orderDetail['cert']['revokedDate']));
                    }
                    $certs[] = $orderParams;
                }
            }
            $this->certs = $certs;
        }
        usort($this->certs, array($this, "sortFunction"));
        return array_reverse($this->certs);
    }

    private function sortFunction($a, $b)
    {
        return strtotime($a["activateddate"]) - strtotime($b["activateddate"]);
    }

    function getStatement($token)
    {
        if (preg_match("/^[a-zA-Z0-9]+$/", $token)) {
            $productCode = $this->getProductCodeFromToken($token);

            if (!$productCode || !ctype_digit($productCode)) {
                $this->errors['getStatement'][] = 'Wrong token.';
                return false;
            }
            $SSLProducts   = new SSLProducts();
            $configuration = new CertumSSLConfiguration();
            $this->setConnectionParams($productCode);
            $op            = $this->client->operationGetStatement();
            $productType   = $SSLProducts->getProductTypeByCode($productCode);
            $op->setLanguage($configuration->getLanguageByProductCode($productType));

            if ($op->call()) {
                return $op->getStatement();
            } else {
                $this->errors['getStatement'][] = $op->getErrorTexts();
                return false;
            }
        } else {
            $this->errors['getStatement'][] = 'Wrong token.';
            return false;
        }
    }

    function generateToken()
    {
        $this->setConnectionParams(null, 1);
        $voucherCode = $this->client->operationGetActivationCode();
        $voucherCode->setActivationCode('261');

        if ($voucherCode->call()) {
            return $this->errors;
        } else {
            $this->errors['generateToken'][] = $voucherCode->getErrorTexts();
            return false;
        }
    }

    function orderDetails($params)
    {
        $this->setConnectionParams(null, $params['relid']);
        $orderDetails = $this->client->operationGetOrderByOrderID();
        $orderDetails->setOrderID($params['orderid']);
        $orderDetails->setOrderStatus(TRUE);
        $orderDetails->setOrderDetails(TRUE);
        $orderDetails->setCertificateDetails(TRUE);
        $response     = $orderDetails->call();

        $responseOrderDetails = array();

        if ($response) {
            if (isset($response->responseHeader->errors)) {
                $errors = $orderDetails->getErrorTexts();
                foreach ($errors as $error) {
                    $this->errors['orderDetails'][] = $orderDetails->getErrorTexts();
                }
            }

            if (is_array($response)) {
                if (isset($response['errors'])) {
                    foreach ($response['errors'] as $errorKey => $errorMessage) {
                        $this->errors['quickOrder'][] = array($errorKey => array('text' => $errorMessage));
                    }
                }
            }

            //Jeśli reissue to robi drugi order i trzeba wziasc ostatni element
            if (is_array($response->orders->Order)) {
                foreach ($response->orders->Order as $order) {
                    //Ale przed trzeba order details wyciagnac z pierwszego w drugim nie ma
                    if ($response->orders->Order[0]->orderDetails) {
                        foreach ($response->orders->Order[0]->orderDetails as $detailsSectionName => $detailsSectionValue) {
                            foreach ($detailsSectionValue as $detailsSectionParamName => $detailsSectionParamValue) {
                                $orderParams[$detailsSectionName][$detailsSectionParamName] = $detailsSectionParamValue;
                            }
                        }
                    }
                    $orderParams['orderdefault']['status']  = $order->orderStatus->orderStatus;
                    $orderParams['orderdefault']['orderID'] = $order->orderStatus->orderID;
                    $orderParams['orderdefault']['reissue'] = $order->reissue;

                    if ($order->certificateDetails) {
                        foreach ($order->certificateDetails as $certParamName => $certParamValue) {
                            if ($certParamName == 'revokedDate') {
                                $orderParams['cert'][$certParamName]         = date("Y-m-d H:i:s", strtotime($certParamValue));
                                $orderParams['orderdefault'][$certParamName] = date("Y-m-d H:i:s", strtotime($certParamValue));
                            } else {
                                $orderParams['cert'][$certParamName] = $certParamValue;
                            }
                        }
                    }
                    $responseOrderDetails[] = $orderParams;
                }
            } else {
                if ($response->orders->Order->orderDetails) {
                    foreach ($response->orders->Order->orderDetails as $detailsSectionName => $detailsSectionValue) {
                        foreach ($detailsSectionValue as $detailsSectionParamName => $detailsSectionParamValue) {
                            if ($detailsSectionParamName == 'revokedDate') {
                                $orderParams[$detailsSectionName][$detailsSectionParamName] = date("Y-m-d H:i:s", strtotime($detailsSectionParamValue));
                                $orderParams['orderdefault'][$certParamName]                = date("Y-m-d H:i:s", strtotime($certParamValue));
                            } else {
                                $orderParams[$detailsSectionName][$detailsSectionParamName] = $detailsSectionParamValue;
                            }
                        }
                    }
                }
                $orderParams['orderdefault']['status']  = $response->orders->Order->orderStatus->orderStatus;
                $orderParams['orderdefault']['orderID'] = $response->orders->Order->orderStatus->orderID;
                $orderParams['orderdefault']['reissue'] = $response->orders->Order->reissue;

                if (isset($response->orders->Order->certificateDetails)) {
                    foreach ($response->orders->Order->certificateDetails as $certParamName => $certParamValue) {
                        if ($certParamName == 'revokedDate') {
                            $orderParams['cert'][$certParamName]         = date("Y-m-d H:i:s", strtotime($certParamValue));
                            $orderParams['orderdefault'][$certParamName] = date("Y-m-d H:i:s", strtotime($certParamValue));
                        } else {
                            $orderParams['cert'][$certParamName] = $certParamValue;
                        }
                    }
                }
                $responseOrderDetails[] = $orderParams;
            }
            return $responseOrderDetails;
        } else {
            $this->errors['orderDetails'][] = $orderDetails->getErrorTexts();
            return false;
        }
    }

    function certficateDetails($params)
    {
        $certyficateDetails = $this->client->operationGetCertificate();
        $certyficateDetails->setOrderID($params['orderID']);
        $response           = $certyficateDetails->call();

        if ($response) {
            if (isset($response->responseHeader->errors)) {
                $errors = $certyficateDetails->getErrorTexts();
                foreach ($errors as $error) {
                    $this->errors['certyficateDetails'][] = $error;
                }
                return false;
            }
            return $response;
        } else {
            $this->errors['certyficateDetails'][] = $certyficateDetails->getErrorTexts();
            return false;
        }
    }

    private function decodeProductCode($a = '', $b = '', $c = '')
    {
        $alpha = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
        $ai    = (int) strpos($alpha, $a);
        $bi    = (int) strpos($alpha, $b);
        $ci    = (int) strpos($alpha, $c);
        $i     = ((($ci - $ai - $bi) % 62) + 62) % 62;
        return $alpha{$i};
    }

    public function getProductCodeFromToken($token)
    {
        if (strlen($token) < 11) {
            return false;
        }
        $typ = $this->decodeProductCode($token{1}, $token{3}, $token{2}).
            $this->decodeProductCode($token{5}, $token{7}, $token{6}).
            $this->decodeProductCode($token{9}, $token{11}, $token{10});

        return $typ;
    }

    function quickOrder($params)
    {
        $quickOrder = $this->client->operationQuickOrder();
        $quickOrder->setCSR($params['csr']);
        $quickOrder->setActivationCode($params['productvoucherCode']);

        if (isset($params['email']) && $params['email'] != "") {
            $quickOrder->setEmail($params['email']);
        }

        // Get or set unique id for user
        $userUniqueIdTable = new MGUsersUniqid(array('_tableName' => 'certumssl_users_uniqid', 'userId' => $this->userId));
        $userUniqueId = $userUniqueIdTable->getUniqueId($this->userId);

        if (empty($userUniqueId)) {
            $userUniqueId = uniqid($params['requestorInfo']['firstName'].$params['requestorInfo']['lastName']);
            $userUniqueIdTable->insertUniqueId($userUniqueId);
        }

        $quickOrder->setCustomer($userUniqueId);
        $productCode = $this->getProductCodeFromToken($params['productvoucherCode']);
        $quickOrder->setProductCode($productCode);

        $quickOrder->setRequestorInfo(
            $params['requestorInfo']['firstName'], $params['requestorInfo']['lastName'], $params['requestorInfo']['email'], $params['requestorInfo']['phone'], $params['requestorInfo']['country'],
            $params['requestorInfo']['postalCode'], $params['requestorInfo']['city'], $params['requestorInfo']['address']
        );

        if (isset($params['cert_contactorg_name']) && isset($params['taxIdentificationNumber']) && trim($params['cert_contactorg_name']) != '' && $params['taxIdentificationNumber'] != '') {
            $quickOrder->setOrganizationInfo($params['cert_contactorg_name'], $params['taxIdentificationNumber']);
        }

        $SSLProducts    = new SSLProducts();
        $sslProductType = $SSLProducts->getProductTypeByCode($productCode);

        if (!is_array($params['domains'])) {
            $domainName          = $params['domains'];
            $params['domains']   = array();
            $params['domains'][] = $domainName;
            $params['domains'][] = 'www.'.$domainName;
        } else {
            $newDomains = array();
            foreach ($params['domains'] as $domainName) {
                $newDomains[] = $domainName;
                if (strpos($domainName, 'www.') === false) {
                    $newDomains[] = 'www.'.$domainName;
                }
            }
            $params['domains'] = $newDomains;
        }

        //Dla certyfikatow ID nalezy z zadania usunac sekcje <approvers> i <SANEntries>.
        if ($sslProductType != "professional_id" && $sslProductType != "enterprise_id" && $sslProductType != "basic_id") {
            $quickOrder->addSANEntry($params['domains']);
            if (is_array($params['domains'])) {
                foreach ($params['domains'] as $domain) {
                    if (strpos($domain, 'www.') === false) {
                        $quickOrder->addApprover($domain, NULL, 'FILE');
                    }
                }
            } else {
                $quickOrder->addApprover($params['FQDN'], NULL, 'FILE');
            }
        }

        $response = $quickOrder->call();

        if ($response) {
            if (isset($response->responseHeader->errors)) {
                $errors = $quickOrder->getErrorTexts();
                foreach ($errors as $error) {
                    $this->errors['quickOrder'][] = $error['text'];
                }
            }
            if (is_array($response)) {
                if (isset($response['errors'])) {
                    foreach ($response['errors'] as $errorKey => $errorMessage) {
                        $this->errors['quickOrder'][] = array($errorKey => array('text' => $errorMessage));
                    }
                }
            }

            return $response;
        } else {
            $errors = $quickOrder->getErrorTexts();
            foreach ($errors as $error) {
                $this->errors['quickOrder'][] = $error['text'];
            }
            return false;
        }
    }

    function cancelOrder($params)
    {
        $cancelOrder = $this->client->operationCancelOrder();
        $cancelOrder->setOrderID($params['orderId']);
        $response    = $cancelOrder->call();

        if ($response) {
            if (isset($response->responseHeader->errors)) {
                $errors = $cancelOrder->getErrorTexts();
                foreach ($errors as $error) {
                    $this->errors['cancelOrder'][] = $cancelOrder->getErrorTexts();
                }
                return false;
            }
            return $response;
        } else {
            $this->errors['cancelOrder'][] = $cancelOrder->getErrorTexts();
            return false;
        }
    }

    function revokeCertificate($params)
    {
        $revokeCertificate = $this->client->operationRevokeCertificate();
        $revokeCertificate->setSerialNumber($params['serialNumber']);
        $response          = $revokeCertificate->call();

        if ($response) {
            if (isset($response->responseHeader->errors)) {
                $errors = $revokeCertificate->getErrorTexts();
                foreach ($errors as $error) {
                    $this->errors['revokeCertificate'][] = $revokeCertificate->getErrorTexts();
                }
                return false;
            }
            return $response;
        } else {
            $this->errors['revokeCertificate'][] = $revokeCertificate->getErrorTexts();
            return false;
        }
    }

    function reissueCertificate($params)
    {

        $reissueCertificate = $this->client->operationReissueCertificate();
        $reissueCertificate->setSerialNumber($params['serialNumber']);
        $reissueCertificate->setCSR($params['csr']);
        $response           = $reissueCertificate->call();

        if ($response) {
            if (isset($response->responseHeader->errors)) {
                $errors = $reissueCertificate->getErrorTexts();
                foreach ($errors as $error) {
                    $this->errors['reissueCertificate'][] = $reissueCertificate->getErrorTexts();
                }
                return false;
            }
            return $response;
        } else {
            $this->errors['reissueCertificate'][] = $reissueCertificate->getErrorTexts();
            return false;
        }
    }

    function verifyDomain($params)
    {

        $verifyDomain = $this->client->operationVerifyDomain();
        $verifyDomain->setCode($params['code']);
        $response     = $verifyDomain->call();
        if ($response) {
            if (isset($response->responseHeader->errors)) {
                $errors = $verifyDomain->getErrorTexts();
                foreach ($errors as $error) {
                    $this->errors['verifyDomain'][] = $verifyDomain->getErrorTexts();
                }
                return false;
            }
            return $response;
        } else {
            $this->errors['verifyDomain'][] = $verifyDomain->getErrorTexts();
            return false;
        }
    }

    function getAPIAccountParamsByVoucher($voucherCode)
    {
        $SSLProducts      = new SSLProducts();
        $configuration    = new CertumSSLConfiguration();
        $productCode      = $this->getProductCodeFromToken($voucherCode);
        $productType      = $SSLProducts->getProductTypeByCode($productCode);
        $apiAccountParams = $configuration->getAccountByPrductType($productType);
        return $apiAccountParams;
    }
}