#!/bin/bash

#DB Config
dbHost="127.0.0.1";
dbName="certum_ssl";
dbUserName="certum_ssl";
dbPassword=`< /dev/urandom tr -dc _A-Z-a-z-0-9 | head -c${1:-32};echo;`

#Config File
configFile='/usr/local/cpanel/etc/CertumSSL.ini';
cronFile='/usr/local/cpanel/whostmgr/docroot/cgi/CertumSSL/CertumSSLCron.php';

declare -a copyFiles=(
    'CertumSSLWHM' '/usr/local/cpanel/whostmgr/docroot/cgi/CertumSSL'
    'CertumSSLCPanel' '/usr/local/cpanel/base/frontend/x3/CertumSSL'
    'CertumSSLCPanel' '/usr/local/cpanel/base/frontend/paper_lantern/CertumSSL'
    'CertumSSLIncludes' '/usr/local/cpanel/share/CertumSSL'
    'CertumSSLBinAdmin' '/usr/local/cpanel/bin/admin/CertumSSL'
    'CertumSSLCPanelAPI' '/usr/local/cpanel/Cpanel/API/'
)

declare -a chmodFiles=(
    '/usr/local/cpanel/logs/CertumSSL' '722'
    '/usr/local/cpanel/base/frontend/x3/CertumSSL' '755'
    '/usr/local/cpanel/whostmgr/docroot/cgi/CertumSSL' '755'
    '/usr/local/cpanel/bin/admin/CertumSSL' '644'
    '/usr/local/cpanel/bin/admin/CertumSSL/getadmin' '755'
)

# Detect paths
MYSQL=$(which mysql)
CurrentDIR=${PWD}

if [ -f  "$configFile" ]
then
    installed=1;
    dbHost=`cat $configFile|grep "db_host"|cut -d'=' -f2 |tr -d '\r'| cut -d "'" -f 2`;
    dbName=`cat $configFile|grep "db_user"|cut -d'=' -f2 |tr -d '\r'| cut -d "'" -f 2`;
    dbUserName=`cat $configFile|grep "db_name"|cut -d'=' -f2 |tr -d '\r'| cut -d "'" -f 2`;
    dbPassword=`cat $configFile|grep "db_pass"|cut -d'=' -f2 |tr -d '\r'| cut -d "'" -f 2`;
fi;

if [ "$installed" = "1" ]
then
    echo -n "
Select Action:
    1 - Update Plugin
    2 - Uninstall Plugin
:";
    read option;
    case "$option" in
        "1") action='update'; actionLBL='Update Plugin' ;;
        "2") action='uninstall'; actionLBL='Uninstall Plugin' ;;
    esac;
else
    actionLBL='Install Plugin';
    action='install';
fi;

if [[ -z "$action" ]]
then
    echo "Exiting";
    exit;
fi;


echo -n "Are you sure you want to $action ? (yes/no):";

read selected;

if [[ "$selected" != "yes" ]]
then
    echo "Exiting";
    exit;
fi;

data=`$MYSQL -h "$dbHost" -u 'root' -e 'SHOW Databases;'`;

if [ ! "$?" -eq 0 ]
then
    echo -n "Please provide MySQL admin username [root]";
    read $mysqlRootUser;

    echo -n "Please provide MySQL admin password";
    read $mysqlRootPassword;
fi;

case "$action" in
    "install")
            SQL="create database $dbName;CREATE USER $dbUserName@localhost IDENTIFIED BY '$dbPassword';GRANT ALL ON $dbName.* TO $dbUserName@localhost;";
            if [ -z "$mysqlRootPassword" ]
            then
                $MYSQL -h "$dbHost" -u root -e "$SQL"
                $MYSQL -h "$dbHost" -u root "$dbName" < ./InstallFiles/install.sql
            else
                $MYSQL -h "$dbHost" -u "$dbUserName" --password="$dbPassword" "$dbName" -e "$SQL"
                $MYSQL -h "$dbHost" -u "$dbUserName" --password="$dbPassword" "$dbName" < ./InstallFiles/install.sql
            fi;

            echo "
            db_host='$dbHost'
            db_user='$dbUserName'
            db_name='$dbName'
            db_pass='$dbPassword'
            " > "$configFile"

            chmod 600 "$configFile"

            for dir in "${copyFiles[@]}"
            do
                if([[ -z "$source" ]])
                then
                    source=$dir;
                else
                    if [ ! -d "$dir" ]
                    then
                        mkdir "$dir"
                    fi;
                    cp -fR "$CurrentDIR/$source/." "$dir"
                    source='';
                fi;
            done

            source='';
            for dir in "${chmodFiles[@]}"
            do
                if([[ -z "$source" ]])
                then
                    source=$dir;
                else
                    if [[ ! -d "$source" && ! -f "$source" ]]
                    then
                        mkdir "$source"
                    fi;
                    chmod -R "$dir" "$source"
                    source='';
                fi;
            done

            if [ ! -f "/root/.accesshash" ]; 
            then
                echo -n "Generating access hash for the WHM API.."
                export REMOTE_USER="root"
                /usr/local/cpanel/bin/realmkaccesshash
                chmod 660 /root/.accesshash
                echo "Done"
            fi

            if [ -e "$CurrentDIR/InstallFiles/CertumSSLWHMPlugin.conf" ]
            then
                /usr/local/cpanel/bin/register_appconfig "$CurrentDIR/InstallFiles/CertumSSLWHMPlugin.conf"
                cp "$CurrentDIR/InstallFiles/certum_admin_icon.png" /usr/local/cpanel/whostmgr/docroot/addon_plugins
            fi;

            if [ -e "$CurrentDIR/InstallFiles/CertumSSL.cpanelplugin" ]
            then
                cp "$CurrentDIR/InstallFiles/CertumSSL.cpanelplugin" /tmp/CertumSSL.cpanelplugin
                cat "$CurrentDIR/InstallFiles/certum_icon.png" | base64 >> /tmp/CertumSSL.cpanelplugin
                /usr/local/cpanel/bin/register_cpanelplugin /tmp/CertumSSL.cpanelplugin
                rm -f /tmp/CertumSSL.cpanelplugin
            fi;

            if [ -e "$CurrentDIR/InstallFiles/install.json" ]
            then 
                tar -C "$CurrentDIR/InstallFiles/" -cvjf /tmp/CertumSSL.tar.bz2 install.json certum_icon.png
                /usr/local/cpanel/scripts/install_plugin /tmp/CertumSSL.tar.bz2 --theme paper_lantern
                rm -f /tmp/CertumSSL.tar.bz2
            fi;

            command="/usr/local/cpanel/3rdparty/bin/php $cronFile"
            job="*/5 * * * * $command"
            cat <(fgrep -i -v "$command" <(crontab -l)) <(echo "$job") | crontab -
            echo "Cron Job Added successful";
    ;;
    "uninstall")
        SQL="DROP DATABASE $dbName; DROP USER $dbUserName@localhost;"

        if [ -z "$mysqlRootPassword" ]
        then
            $MYSQL -h "$dbHost" -u root -e "$SQL"
        else
            $MYSQL -h "$dbHost" -u '"$dbUserName"' --password="$dbPassword" -e "$SQL"
        fi;

        for dir in "${copyFiles[@]}"
        do
            if([[ -z "$source" ]])
            then
                source=$dir;
            else
                if [ -d "$dir" ]
                then
                    if [ "$dir" == "/usr/local/cpanel/Cpanel/API/" ]
                    then
                        if [ -f "/usr/local/cpanel/Cpanel/API/CertumSSL.pm" ]
                        then
                            rm -f /usr/local/cpanel/Cpanel/API/CertumSSL.pm
                        fi;
                    elif [ -d "$dir" ] && [ "$dir" != "/usr/local/cpanel/Cpanel/API/" ]
                    then
                        rm -rf "$dir"
                    fi;
                fi;
                source='';
            fi;
        done

        rm "$configFile";

        if [ -e "$CurrentDIR/InstallFiles/CertumSSLWHMPlugin.conf" ]
        then
            /usr/local/cpanel/bin/unregister_appconfig "$CurrentDIR/InstallFiles/CertumSSLWHMPlugin.conf"
        fi;

        echo "Removing module in cPanel x3 theme..."
        if [ -e "$CurrentDIR/InstallFiles/CertumSSL.cpanelplugin" ]
        then
            cp "$CurrentDIR/InstallFiles/CertumSSL.cpanelplugin" /tmp/CertumSSL.cpanelplugin
            cat "$CurrentDIR/InstallFiles/cer.png" | base64 >> /tmp/CertumSSL.cpanelplugin
            /usr/local/cpanel/bin/unregister_cpanelplugin /tmp/CertumSSL.cpanelplugin
            rm -f /tmp/CertumSSL.cpanelplugin
        fi;

        if [ -e "/usr/local/cpanel/base/frontend/x3/dynamicui/dynamicui_CertumSSL.conf" ]
        then
            rm -f /usr/local/cpanel/base/frontend/x3/dynamicui/dynamicui_CertumSSL.conf
        fi;

        echo "Removing module in cPanel paper_lanter theme..."
        if [ -e "/usr/local/cpanel/base/frontend/paper_lantern/dynamicui/dynamicui_CertumSSL.conf" ]
        then
            rm -f /usr/local/cpanel/base/frontend/paper_lantern/dynamicui/dynamicui_CertumSSL.conf
        fi;
    ;;
    "update")
        $MYSQL -h "$dbHost" -u "$dbUserName" --password="$dbPassword" "$dbName" < ./InstallFiles/install.sql

        for dir in "${copyFiles[@]}"
        do
            if([[ -z "$source" ]])
            then
                source=$dir;
            else
                if [ ! -d "$dir" ]
                then
                    mkdir "$dir"
                fi;
                cp -fR "$CurrentDIR/$source/." "$dir"
                source='';
            fi;
        done

        source='';
        for dir in "${chmodFiles[@]}"
        do
            if([[ -z "$source" ]])
            then
                source=$dir;
            else
                if [[ ! -d "$source" && ! -f "$source" ]]
                then
                    mkdir "$source"
                fi;
                chmod -R "$dir" "$source"
                source='';
            fi;
        done

        if [ -e "$CurrentDIR/InstallFiles/CertumSSLWHMPlugin.conf" ]
        then
            /usr/local/cpanel/bin/register_appconfig "$CurrentDIR/InstallFiles/CertumSSLWHMPlugin.conf"
        fi;

        if [ -e "$CurrentDIR/InstallFiles/CertumSSL.cpanelplugin" ]
        then
            cp "$CurrentDIR/InstallFiles/CertumSSL.cpanelplugin" /tmp/CertumSSL.cpanelplugin
            cat "$CurrentDIR/InstallFiles/certum_icon.png" | base64 >> /tmp/CertumSSL.cpanelplugin
            /usr/local/cpanel/bin/register_cpanelplugin /tmp/CertumSSL.cpanelplugin
            rm -f /tmp/CertumSSL.cpanelplugin
        fi;

        if [ -e "$CurrentDIR/InstallFiles/install.json" ]
        then 
            tar -C "$CurrentDIR/InstallFiles/" -cvjf /tmp/CertumSSL.tar.bz2 install.json certum_icon.png
            /usr/local/cpanel/scripts/install_plugin /tmp/CertumSSL.tar.bz2
            rm -f /tmp/CertumSSL.tar.bz2
        fi;

        currentCrons=`crontab -l |grep $cronFile`

        if [ -z "$currentCrons" ]
        then

            echo -n "Missing Cron Job Entry, are you want to add them ? (yes/no):";

            read selected;

            if [[ "$selected" == "yes" ]]
            then
                command="/usr/local/cpanel/3rdparty/bin/php $cronFile"
                job="*/5 * * * * $command"
                cat <(fgrep -i -v "$command" <(crontab -l)) <(echo "$job") | crontab -
                command="/usr/local/cpanel/3rdparty/bin/php /usr/local/cpanel/whostmgr/docroot/cgi/CertumSSL/CertumSSLCron.php"
                job="0 0 * * * $command"
                cat <(fgrep -i -v "$command" <(crontab -l)) <(echo "$job") | crontab -
                echo "Cron Job Added successful";
            fi;
        fi;
    ;;
esac;

echo "Action: $actionLBL finished successfull";